//+------------------------------------------------------------------+
//|                                                    Pinbar LC.mq5 |
//|                   Copyright 2009-2013, MetaQuotes Software Corp. |
//|                                              http://www.mql5.com |
//+------------------------------------------------------------------+
#property copyright "Copyright 2009-2013, MetaQuotes Software Corp."
#property link      "http://www.mql5.com"
#property version   "1.00"

#include <Trade\Trade.mqh>

input double MaximumRisk         =0.1;    // Maximum Risk in percentage
input double DecreaseFactor      =3;      // Descrease factor
//---
input uint   inp_pb_min_shdw     = 40;    // Pinbar min shadow, point
input uint   inp_pb_max_OC       = 20;    // Pinbar max OC, point
input double inp_pb_min_ratio    = 2.0;   // Pinbar shadow to OC min ratio
//---
input uint   inp_sl_pp = 300;             // SL, points (0 for OFF)
input uint   inp_tp_pp = 500;             // TP, points (0 for OFF)
//---
input ENUM_TIMEFRAMES   BaseTF=PERIOD_M1; // LC Base Period
input bool   LC_on = true;                // LC mode ON
input int    LC_shift = 0;                // LC shift
//---      
#include <liquidchart.mqh>
//+------------------------------------------------------------------+
//| Calculate optimal lot size                                       |
//+------------------------------------------------------------------+
double TradeSizeOptimized(void)
  {
   double price=0.0;
   double margin=0.0;
//--- select lot size
   if(!SymbolInfoDouble(_Symbol,SYMBOL_ASK,price))
      return(0.0);
   if(!OrderCalcMargin(ORDER_TYPE_BUY,_Symbol,1.0,price,margin))
      return(0.0);
   if(margin<=0.0)
      return(0.0);

   double lot=NormalizeDouble(AccountInfoDouble(ACCOUNT_FREEMARGIN)*MaximumRisk/margin,2);
//--- calculate number of losses orders without a break
   if(DecreaseFactor>0)
     {
      //--- select history for access
      HistorySelect(0,TimeCurrent());
      //---
      int    orders=HistoryDealsTotal();  // total history deals
      int    losses=0;                    // number of losses orders without a break

      for(int i=orders-1;i>=0;i--)
        {
         ulong ticket=HistoryDealGetTicket(i);
         if(ticket==0)
           {
            Print("HistoryDealGetTicket failed, no trade history");
            break;
           }
         //--- check symbol
         if(HistoryDealGetString(ticket,DEAL_SYMBOL)!=_Symbol)
            continue;
         //--- check profit
         double profit=HistoryDealGetDouble(ticket,DEAL_PROFIT);
         if(profit>0.0)
            break;
         if(profit<0.0)
            losses++;
        }
      //---
      if(losses>1)
         lot=NormalizeDouble(lot-lot*losses/DecreaseFactor,1);
     }
//--- normalize and check limits
   double stepvol=SymbolInfoDouble(_Symbol,SYMBOL_VOLUME_STEP);
   lot=stepvol*NormalizeDouble(lot/stepvol,0);

   double minvol=SymbolInfoDouble(_Symbol,SYMBOL_VOLUME_MIN);
   if(lot<minvol)
      lot=minvol;

   double maxvol=SymbolInfoDouble(_Symbol,SYMBOL_VOLUME_MAX);
   if(lot>maxvol)
      lot=maxvol;
//--- return trading volume
   return(lot);
  }
//+------------------------------------------------------------------+
//| Detecting new bar                                                |
//+------------------------------------------------------------------+
bool newbar(datetime t)
  {
   static datetime t_prev=0;
   if(t!=t_prev)
     {
      t_prev=t;
      return(true);
     }
   return(false);
  }
//+------------------------------------------------------------------+
//| Check for pinbar                                                 |
//+------------------------------------------------------------------+
int IsPinbar(MqlRates &r[])
  {//  r[]   4 
   if(ArraySize(r)<4)
      return(0);
   if(r[0].open<r[0].close && r[2].open>r[2].close && r[1].high>MathMax(r[0].high,r[2].high))
     {//  
      double oc=MathAbs(r[1].open-r[1].close)/_Point;
      if(oc>inp_pb_max_OC)
         return(0);
      double shdw=(r[1].high-MathMax(r[1].open,r[1].close))/_Point;
      if(shdw<inp_pb_min_shdw)
         return(0);
      if(oc!=0)
        {
         if((shdw/oc)<inp_pb_min_ratio)
            return(0);
        }
      return(1);
     }
   else if(r[0].open>r[0].close && r[2].open<r[2].close && r[1].low<MathMin(r[0].low,r[2].low))
     {//  
      double oc=MathAbs(r[1].open-r[1].close)/_Point;
      if(oc>inp_pb_max_OC)
         return(0);
      double shdw=(MathMin(r[1].open,r[1].close)-r[1].low)/_Point;
      if(shdw<inp_pb_min_shdw)
         return(0);
      if(oc!=0)
        {
         if((shdw/oc)<inp_pb_min_ratio)
            return(0);
        }
      return(-1);
     }
   return(0);
  }
//+------------------------------------------------------------------+
//| Check for open position conditions                               |
//+------------------------------------------------------------------+
void CheckForOpen(void)
  {
   MqlRates rt[4];
//--- go trading only for first ticks of new bar
   int copied;
   if(LC_on==true)
     {
      int shift=LC_shift;
      copied=GetRatesLC(0,4,rt,BaseTF,shift);
     }
   else
      copied=CopyRates(_Symbol,_Period,0,4,rt);
   if(copied!=4)
     {
      Print("CopyRates of ",_Symbol," failed, no history");
      return;
     }
   if(newbar(rt[3].time)==false)
      return;
//--- check signals
   ENUM_ORDER_TYPE signal=WRONG_VALUE;
   int pb=IsPinbar(rt);
   if(pb==1)//   (  )
      signal=ORDER_TYPE_SELL; // sell conditions
   else if(pb==-1)//   (  )
      signal=ORDER_TYPE_BUY;  // buy conditions
//--- additional checking
   if(signal!=WRONG_VALUE)
      if(TerminalInfoInteger(TERMINAL_TRADE_ALLOWED))
         if(Bars(_Symbol,_Period)>100)
           {
            double sl=0,tp=0,p=0;
            double ask=SymbolInfoDouble(_Symbol,SYMBOL_ASK);
            double bid=SymbolInfoDouble(_Symbol,SYMBOL_BID);
            if(signal==ORDER_TYPE_SELL)
              {
               p=bid;
               if(inp_sl_pp!=0)
                  sl=NormalizeDouble(ask+inp_sl_pp*_Point,_Digits);
               if(inp_tp_pp!=0)
                  tp=NormalizeDouble(ask-inp_sl_pp*_Point,_Digits);
              }
            else
              {
               p=ask;
               if(inp_sl_pp!=0)
                  sl=NormalizeDouble(bid-inp_sl_pp*_Point,_Digits);
               if(inp_tp_pp!=0)
                  tp=NormalizeDouble(bid+inp_sl_pp*_Point,_Digits);
              }
            CTrade trade;
            trade.PositionOpen(_Symbol,signal,TradeSizeOptimized(),p,sl,tp);
           }
//---
  }
//+------------------------------------------------------------------+
//| Check for close position conditions                              |
//+------------------------------------------------------------------+
void CheckForClose(void)
  {
   MqlRates rt[4];
//--- go trading only for first ticks of new bar
   int copied;
   if(LC_on==true)
     {
      int shift=LC_shift;
      copied=GetRatesLC(0,4,rt,BaseTF,shift);
     }
   else
      copied=CopyRates(_Symbol,_Period,0,4,rt);
   if(copied!=4)
     {
      Print("CopyRates of ",_Symbol," failed, no history");
      return;
     }
   if(newbar(rt[3].time)==false)
      return;
//--- positions already selected before
   bool signal=false;
   long type=PositionGetInteger(POSITION_TYPE);
   int pb=IsPinbar(rt);
   if(type==(long)POSITION_TYPE_BUY && pb==1)
      signal=true;
   if(type==(long)POSITION_TYPE_SELL && pb==-1)
      signal=true;
//--- additional checking
   if(signal)
      if(TerminalInfoInteger(TERMINAL_TRADE_ALLOWED))
         if(Bars(_Symbol,_Period)>100)
           {
            CTrade trade;
            trade.PositionClose(_Symbol,3);
           }
//---
  }
//+------------------------------------------------------------------+
//| Expert initialization function                                   |
//+------------------------------------------------------------------+
int OnInit(void)
  {
//---       
   int k=PeriodSeconds()/PeriodSeconds(BaseTF);
   if(k<1)
     {
      Comment("   ");
      return(INIT_PARAMETERS_INCORRECT);
     }
   if(LC_shift<-2 || LC_shift>=k)
     {
      Comment("  ");
      return(INIT_PARAMETERS_INCORRECT);
     }
//---
   return(INIT_SUCCEEDED);
  }
//+------------------------------------------------------------------+
//| Expert tick function                                             |
//+------------------------------------------------------------------+
void OnTick(void)
  {
//---
   if(PositionSelect(_Symbol))
      CheckForClose();
   else
      CheckForOpen();
//---
  }
//+------------------------------------------------------------------+
//| Expert deinitialization function                                 |
//+------------------------------------------------------------------+
void OnDeinit(const int reason)
  {
  }
//+------------------------------------------------------------------+
